# Generated by Django 3.2.7 on 2021-09-13 15:16

from django.db import migrations
from django.db.models import Count


def deduplicate_attempts(apps, schema_editor):
    AccessAttempt = apps.get_model("axes", "AccessAttempt")
    duplicated_attempts = (
        AccessAttempt.objects.values("username", "user_agent", "ip_address")
        .annotate(Count("id"))
        .order_by()
        .filter(id__count__gt=1)
    )

    for attempt in duplicated_attempts:
        redundant_attempts = AccessAttempt.objects.filter(
            username=attempt["username"],
            user_agent=attempt["user_agent"],
            ip_address=attempt["ip_address"],
        )[1:]
        for redundant_attempt in redundant_attempts:
            redundant_attempt.delete()


class Migration(migrations.Migration):

    dependencies = [
        ("axes", "0006_remove_accesslog_trusted"),
    ]

    operations = [
        migrations.RunPython(deduplicate_attempts),
        migrations.AlterUniqueTogether(
            name="accessattempt",
            unique_together={("username", "ip_address", "user_agent")},
        ),
    ]



"""
Axes utility functions that are publicly available.

This module is separate for historical reasons
and offers a backwards compatible import path.
"""

from logging import getLogger
from typing import Optional

from django.http import HttpRequest

from axes.conf import settings
from axes.handlers.proxy import AxesProxyHandler
from axes.helpers import get_client_ip_address

log = getLogger(__name__)


def reset(
    ip: Optional[str] = None, username: Optional[str] = None, ip_or_username=False
) -> int:
    """
    Reset records that match IP or username, and return the count of removed attempts.

    This utility method is meant to be used from the CLI or via Python API.
    """

    return AxesProxyHandler.reset_attempts(
        ip_address=ip, username=username, ip_or_username=ip_or_username
    )


def reset_request(request: HttpRequest) -> int:
    """
    Reset records that match IP or username, and return the count of removed attempts.

    This utility method is meant to be used from the CLI or via Python API.
    """

    ip: Optional[str] = get_client_ip_address(request)
    username = request.GET.get("username", None)

    ip_or_username = settings.AXES_LOCK_OUT_BY_USER_OR_IP
    if settings.AXES_ONLY_USER_FAILURES:
        ip = None
    elif not (
        settings.AXES_LOCK_OUT_BY_USER_OR_IP
        or settings.AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP
    ):
        username = None

    if not ip and not username:
        return 0
        # We don't want to reset everything, if there is some wrong request parameter

    # if settings.AXES_USE_USER_AGENT:
    # TODO: reset based on user_agent?
    return reset(ip, username, ip_or_username)



from logging import getLogger

from django import apps
from pkg_resources import get_distribution

log = getLogger(__name__)


class AppConfig(apps.AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "axes"
    initialized = False

    @classmethod
    def initialize(cls):
        """
        Initialize Axes logging and show version information.

        This method is re-entrant and can be called multiple times.
        It displays version information exactly once at application startup.
        """

        if cls.initialized:
            return
        cls.initialized = True

        # Only import settings, checks, and signals one time after Django has been initialized
        from axes.conf import settings  # noqa
        from axes import checks, signals  # noqa

        # Skip startup log messages if Axes is not set to verbose
        if settings.AXES_VERBOSE:
            if settings.AXES_ONLY_USER_FAILURES:
                mode = "blocking by username only"
            elif settings.AXES_LOCK_OUT_BY_COMBINATION_USER_AND_IP:
                mode = "blocking by combination of username and IP"
            elif settings.AXES_LOCK_OUT_BY_USER_OR_IP:
                mode = "blocking by username or IP"
            else:
                mode = "blocking by IP only"
            log.info(
                "AXES: BEGIN version %s, %s",
                get_distribution("django-axes").version,
                mode,
            )

    def ready(self):
        self.initialize()



from django.apps.registry import apps
from django.db import connection
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.executor import MigrationExecutor
from django.db.migrations.state import ProjectState

from axes.models import AccessAttempt, AccessLog, AccessFailureLog
from tests.base import AxesTestCase


class ModelsTestCase(AxesTestCase):
    def setUp(self):
        self.failures_since_start = 42

        self.access_attempt = AccessAttempt(
            failures_since_start=self.failures_since_start
        )
        self.access_log = AccessLog()
        self.access_failure_log = AccessFailureLog()

    def test_access_attempt_str(self):
        self.assertIn("Access", str(self.access_attempt))

    def test_access_log_str(self):
        self.assertIn("Access", str(self.access_log))

    def test_access_failure_log_str(self):
        self.assertIn("Failed", str(self.access_failure_log))


class MigrationsTestCase(AxesTestCase):
    def test_missing_migrations(self):
        executor = MigrationExecutor(connection)
        autodetector = MigrationAutodetector(
            executor.loader.project_state(), ProjectState.from_apps(apps)
        )

        changes = autodetector.changes(graph=executor.loader.graph)

        self.assertEqual({}, changes)
